import os
import sys
from typing import Any


class ReadOnly(type):
    def __new__(cls, name, bases, dic):
        _class = super().__new__(cls, name, bases, dic)
        super().__setattr__(
            _class,
            "__ro_cvars__",
            tuple(k for k in dic if not k.startswith("__")),
        )
        return _class

    def __setattr__(self, name: str, value: Any) -> None:
        if name in self.__ro_cvars__:  # type: ignore
            raise AttributeError(
                f"{self.__name__} 类的类属性 {name} 是只读的，无法修改"
            )
        return super().__setattr__(name, value)

    def __instance_setattr(self, name: str, value: Any) -> None:
        if hasattr(self, name):
            raise AttributeError(
                f"{self.__class__.__name__} 类的实例属性 {name} 是只读的，无法修改"
            )
        super(self.__class__, self).__setattr__(name, value)

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        self.__setattr__ = ReadOnly.__instance_setattr  # type: ignore
        return super().__call__(*args, **kwargs)


__version__ = "2.5.14"


class MetaInfo(metaclass=ReadOnly):
    """元信息类

    .. admonition:: 提示
       :class: tip

       一般无需手动实例化该类，多数情况会直接使用本类的属性，或将本类用作类型注解。
    """

    VER: str = __version__
    """melobot 版本

       :meta hide-value:
    """

    PROJ_NAME: str = "melobot"
    """melobot 项目名称

       :meta hide-value:
    """

    PROJ_DESC: str = (
        "A bot development framework with friendly APIs, session control and plugin-supported."
    )
    """melobot 项目描述

       :meta hide-value:
    """

    PROJ_SRC: str = "https://github.com/Meloland/melobot"
    """melobot 项目地址

       :meta hide-value:
    """

    ARGV: list[str] = sys.argv
    """当前运行的 argv

       :meta hide-value:
    """

    OS_NAME: str = os.name
    """当前系统名称

       :meta hide-value:
    """

    PLATFORM: str = sys.platform
    """当前系统平台

       :meta hide-value:
    """

    PY_VER: str = sys.version
    """当前 python 版本

       :meta hide-value:
    """

    PY_INFO: "sys._version_info" = sys.version_info
    """当前 python 信息

       :meta hide-value:
    """

    OS_SEP: str = os.sep

    """当前系统路径分隔符号，如 win 平台下的 "\\"

       :meta hide-value:
    """

    PATH_SEP: str = os.pathsep

    """当前系统路径间的分隔符号，如 win 平台下的 ";"

       :meta hide-value:
    """

    LINE_SEP: str = os.linesep

    """当前系统行尾序列，如 win 平台下的 "\\r\\n"

       :meta hide-value:
    """

    ENV: os._Environ[str] = os.environ
    """当前运行的环境变量

       :meta hide-value:
    """

    @classmethod
    def get_all(cls) -> dict[str, Any]:
        """以字典形式获取所有元信息

        :return: 包含所有元信息的，属性名为键的字典
        """
        return {k: v for k, v in cls.__dict__.items() if not k.startswith("__")}


MELOBOT_LOGO = r"""  __  __      _       ____        _
 |  \/  | ___| | ___ | __ )  ___ | |_
 | |\/| |/ _ \ |/ _ \|  _ \ / _ \| __|
 | |  | |  __/ | (_) | |_) | (_) | |_
 |_|  |_|\___|_|\___/|____/ \___/ \__|
"""
MELOBOT_LOGO_LEN = max(len(_) for _ in MELOBOT_LOGO.split("\n"))



"""MeloBot 是插件化管理、基于异步会话机制的机器人开发框架。

v2 版本目前仅适用于构建与 OneBot 实现对接的机器人服务。

项目源码和文档，请参考：https://github.com/Meloland/melobot
"""

from .base import LogicMode, PriorLevel, SessionRule, User, lock, this_dir, timelimit
from .bot import MeloBot, thisbot
from .context import (
    AttrSessionRule,
    finish,
    msg_args,
    msg_event,
    msg_text,
    pause,
    reply_finish,
    send,
    send_reply,
    send_wait,
    session_store,
)
from .io import ForwardWsConn, HttpConn, ReverseWsConn
from .meta import MetaInfo
from .plugin import BotPlugin
from .utils import CmdArgFormatter, CmdParser, GroupMsgLvlChecker, PrivateMsgLvlChecker

__version__ = MetaInfo.VER



import re

from ..base.abc import BotParser
from ..base.exceptions import BotValueError
from ..base.typing import TYPE_CHECKING, Optional, ParseArgs

if TYPE_CHECKING:
    from .formatter import CmdArgFormatter


class CmdParser(BotParser):
    """命令解析器

    通过解析命令名和命令参数的形式，解析字符串。
    """

    def __init__(
        self,
        cmd_start: str | list[str],
        cmd_sep: str | list[str],
        target: str | list[str],
        formatters: Optional[list[Optional["CmdArgFormatter"]]] = None,
    ) -> None:
        """初始化一个命令解析器

        .. admonition:: 注意
           :class: caution

           - 命令起始符和命令间隔符不允许包含：引号，各种括号，反斜杠，数字，英文，控制字符及各类空白字符。
           - 命令起始符不能是命令间隔符的子序列，反之亦然。

        :param cmd_start: 命令起始符（可以是字符串或字符串列表）
        :param cmd_sep: 命令间隔符（可以是字符串或字符串列表）
        :param target: 匹配的命令名
        :param formatters: 格式化器列表（列表可以包含空值，即此位置的参数无格式化）
        """
        i1 = cmd_start if isinstance(cmd_start, str) else "".join(cmd_start)
        i2 = cmd_sep if isinstance(cmd_sep, str) else "".join(cmd_sep)
        super().__init__(f"{i1}\u0000{i2}")
        self.targets = target if isinstance(target, list) else [target]
        self.formatters = [] if formatters is None else formatters
        self.need_format = (
            True if target is not None and len(self.formatters) > 0 else False
        )

        self.start_tokens = cmd_start if isinstance(cmd_start, list) else [cmd_start]
        self.sep_tokens = cmd_sep if isinstance(cmd_sep, list) else [cmd_sep]
        self.ban_regex = re.compile(r"[\'\"\\\(\)\[\]\{\}\r\n\ta-zA-Z0-9]")
        self._build_parse_regex()

        if self.ban_regex.findall(f"{''.join(cmd_start)}{''.join(cmd_sep)}"):
            raise BotValueError("存在命令解析器不支持的命令起始符，或命令间隔符")

    def _build_parse_regex(self):
        """建立用于命令解析的正则 Pattern 对象，包含命令起始符正则 pattern 和 命令间隔符正则 pattern"""
        temp_regex = re.compile(
            r"([\`\-\=\~\!\@\#\$\%\^\&\*\(\)\_\+\[\]\{\}\|\:\,\.\/\<\>\?])"
        )
        if not len(set(self.sep_tokens) & set(self.start_tokens)):
            self.cmd_sep = [
                temp_regex.sub(r"\\\1", sep_token) for sep_token in self.sep_tokens
            ]
            self.cmd_start = [
                temp_regex.sub(r"\\\1", start_token)
                for start_token in self.start_tokens
            ]
            self.sep_parse_regex = re.compile(rf"{'|'.join(self.cmd_sep)}")
            self.start_parse_regex = re.compile(rf"{'|'.join(self.cmd_start)}")
        else:
            raise BotValueError("命令解析器起始符不能和间隔符重合")

    def _split_string(
        self, string: str, regex: re.Pattern, popFirst: bool = True
    ) -> list[str]:
        """按照指定正则 pattern，对 string 进行分割"""
        # 将复杂的各种分隔符替换为 特殊字符，方便分割
        temp_string = regex.sub("\u0000", string)
        temp_list = re.split("\u0000", temp_string)
        if popFirst:
            temp_list.pop(0)
        return list(filter(lambda x: x != "", temp_list))

    def _parse(self, text: str, textFilter: bool = True) -> dict[str, list[str]]:
        pure_string = text.strip() if textFilter else text
        cmd_strings = self._split_string(pure_string, self.start_parse_regex)
        cmd_seqs = [
            self._split_string(s, self.sep_parse_regex, False) for s in cmd_strings
        ]
        cmd_seqs = list(filter(lambda x: x != [], cmd_seqs))
        cmd_dict: dict[str, list[str]] = {}
        for seq in cmd_seqs:
            if len(seq) == 0:
                continue
            cmd_dict[seq[0]] = seq[1:]
        return cmd_dict

    def parse(self, text: str) -> dict[str, ParseArgs]:
        cmd_dict = self._parse(text)
        return {target: ParseArgs(cmd_dict.get(target)) for target in self.targets}

    def test(
        self, args_dict: dict[str, ParseArgs] | None
    ) -> tuple[bool, Optional[str], Optional[ParseArgs]]:
        # 测试是否匹配。返回三元组：（是否匹配成功，匹配成功的命令名，匹配成功的命令参数）。
        # 最后两个返回值若不存在，则返回 None。
        if args_dict is None:
            return (False, None, None)
        for group_id in args_dict.keys():
            if group_id in self.targets:
                return (True, group_id, args_dict[group_id])
        return (False, None, None)

    async def format(self, group_id: str, args: ParseArgs) -> bool:
        # 格式化命令解析参数
        if hasattr(args, "formatted"):
            return True
        for idx, formatter in enumerate(self.formatters):
            if formatter is None:
                continue
            status = await formatter.format(group_id, args, idx)
            if not status:
                return False
        args.vals = args.vals[: len(self.formatters)]  # type: ignore
        args.formatted = True  # type: ignore
        return True


class CmdParserGen:
    """命令解析器的生成器

    预先存储命令起始符和命令间隔符，指定匹配的命令名后返回一个命令解析器。
    """

    def __init__(self, cmd_start: str | list[str], cmd_sep: str | list[str]) -> None:
        """初始化一个命令解析器的生成器

        .. admonition:: 注意
           :class: caution

           - 命令起始符和命令间隔符不允许包含：引号，各种括号，反斜杠，数字，英文，控制字符及各类空白字符。
           - 命令起始符不能是命令间隔符的子序列，反之亦然。

        :param cmd_start: 命令起始符（可以是字符串或字符串列表）
        :param cmd_sep: 命令间隔符（可以是字符串或字符串列表）
        """
        self.cmd_start = cmd_start
        self.cmd_sep = cmd_sep

    def gen(
        self,
        target: str | list[str],
        formatters: Optional[list[Optional["CmdArgFormatter"]]] = None,
    ) -> CmdParser:
        """生成匹配指定命令名的命令解析器

        :param target: 匹配的命令名
        :param formatters: 格式化器列表（列表可以包含空值，即此位置的参数无格式化选项）
        """
        return CmdParser(self.cmd_start, self.cmd_sep, target, formatters)



import sys

sys.path.append("../../src")


from rich import print

from melobot.utils.parser import CmdParser

p = CmdParser("/", " ", ["ask", "test", "789"])
a = p.parse("/ask")
b = p.parse("/ask test 123")
c = p.parse("asl;jdf;lajf;")


# import json
# import sys
# import time

# from rich import print

# sys.path.append("../../")
# sys.path.append("../../src")
# import asyncio
# import inspect
# from typing import Any

# from melobot import send_reply
# from melobot.base import BotAction
# from melobot.context.action import MsgDelActionArgs, _process_msg
# from melobot.models.msg import image_msg, text_msg
# from melobot.base.tools import is_retcoro


# def timeit(func):
#     def wrapper(*args, **kwargs):
#         start = time.perf_counter()
#         res = func(*args, **kwargs)
#         end = time.perf_counter()
#         print(f"{func.__name__} cost {end - start:0.9f}s")
#         return res

#     return wrapper


# def test2():
#     return 123


# async def test():
#     pass


# async def main():
#     alist = (
#         send_reply,
#         send_reply("123"),
#         asyncio.Future(),
#         asyncio.create_task(test()),
#         lambda: send_reply("123"),
#         test,
#         test(),
#         test2,
#         test2(),
#     )
#     res = (True, False, False, False, True, True, False, False, False)
#     print(tuple(is_retcoro(_, safe_mode=True) for _ in alist) == res)


# asyncio.run(main())

# # start = time.perf_counter()
# # e = MessageEvent(
# #     json.loads(
# #         '{"message_type":"private","sub_type":"friend","message_id":1961549407,"user_id":1574260633,"message":[{"type":"text","data":{"text":"123salkdjf;"}},{"type":"face","data":{"id":"179"}},{"type":"image","data":{"file":"https://multimedia.nt.qq.com.cn/download?appid=1406\u0026fileid=CgoxNTc0MjYwNjMzEhQKN-5rVLOAimuVKucqJjPpzAvSKhjUpgEg_goo6OfInq7BhQM\u0026rkey=CAMSMHAklpltlJtlxKiw85Hus6KgJCAgftqnl9Ha6Ng-Il6rrLoH84r_LeFjMFWyqgV5hA\u0026spec=0","url":"https://multimedia.nt.qq.com.cn/download?appid=1406\u0026fileid=CgoxNTc0MjYwNjMzEhQKN-5rVLOAimuVKucqJjPpzAvSKhjUpgEg_goo6OfInq7BhQM\u0026rkey=CAMSMHAklpltlJtlxKiw85Hus6KgJCAgftqnl9Ha6Ng-Il6rrLoH84r_LeFjMFWyqgV5hA\u0026spec=0","summary":"[\u52A8\u753B\u8868\u60C5]"}}],"raw_message":"123salkdjf;[CQ:face,id=179][CQ:image,file=https://multimedia.nt.qq.com.cn/download?appid=1406\u0026amp;fileid=CgoxNTc0MjYwNjMzEhQKN-5rVLOAimuVKucqJjPpzAvSKhjUpgEg_goo6OfInq7BhQM\u0026amp;rkey=CAMSMHAklpltlJtlxKiw85Hus6KgJCAgftqnl9Ha6Ng-Il6rrLoH84r_LeFjMFWyqgV5hA\u0026amp;spec=0]","font":0,"sender":{"user_id":1574260633,"nickname":"Melorenae\u5F8B\u56DE","sex":"unknown"},"time":1713085889,"self_id":1801297943,"post_type":"message"}'
# #     )
# # )
# # print(f"{e:hexid}, {e:raw}")
# # a = BotAction(MsgDelActionArgs(12312131231233), "13781273712983")
# # print()
# # print(f"{a:hexid}, {a:raw}")
# # print(time.perf_counter() - start)



import asyncio
import json
from abc import ABC, abstractmethod
from copy import deepcopy
from logging import CRITICAL, DEBUG, ERROR, INFO, WARNING, Logger

from .exceptions import BotRuntimeError, BotUtilsError, BotValueError
from .tools import is_retcoro
from .typing import (
    TYPE_CHECKING,
    Any,
    BotLife,
    Callable,
    Coroutine,
    Literal,
    LogicMode,
    ModuleType,
    Optional,
    ParseArgs,
    Type,
    Void,
    VoidType,
)

if TYPE_CHECKING:
    from ..bot.hook import BotHookBus
    from ..controller.dispatcher import BotDispatcher
    from ..controller.responder import BotResponder
    from ..models.event import BotEventBuilder
    from ..plugin.handler import EventHandler


class BaseLogger(Logger):
    """日志器基类

    .. admonition:: 提示
       :class: tip

       一般无需手动实例化该类，多数情况会直接使用本类对象，或将本类用作类型注解。
    """

    LEVEL_MAP = {
        "DEBUG": DEBUG,
        "INFO": INFO,
        "WARNING": WARNING,
        "ERROR": ERROR,
        "CRITICAL": CRITICAL,
    }

    LEVEL_FLAG_NAME = "__LOG_LEVEL_FLAG__"

    def __init__(self, name: str, level: int) -> None:
        super().__init__(name, level)
        setattr(self, BaseLogger.LEVEL_FLAG_NAME, level)

    def check_level_flag(
        self, level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] = "INFO"
    ) -> bool:
        """检查日志器是否可以输出指定日志等级的日志"""
        return BaseLogger.LEVEL_MAP[level] >= getattr(self, BaseLogger.LEVEL_FLAG_NAME)

    LEVEL_CHECK_METH_NAME = check_level_flag.__name__


class AbstractConnector(ABC):
    """抽象连接器类

    .. admonition:: 提示
       :class: tip

       一般无需手动实例化该类，多数情况会直接使用本类对象，或将本类用作类型注解。
    """

    def __init__(self, cd_time: float) -> None:
        super().__init__()
        #: 连接器的日志器
        self.logger: BaseLogger
        #: 是否在 slack 状态
        self.slack: bool = False
        #: 连接器发送行为操作的冷却时间
        self.cd_time = cd_time

        self._used: bool = False
        self._ready_signal = asyncio.Event()
        self._event_builder: Type["BotEventBuilder"]
        self._bot_bus: "BotHookBus"
        self._common_dispatcher: "BotDispatcher"
        self._resp_dispatcher: "BotResponder"

    @abstractmethod
    async def __aenter__(self):
        raise NotImplementedError

    @abstractmethod
    async def __aexit__(
        self, exc_type: Type[Exception], exc_val: Exception, exc_tb: ModuleType
    ) -> bool:
        if exc_type is None:
            return True
        elif exc_type == asyncio.CancelledError:
            return True
        else:
            return False

    def _set_ready(self) -> None:
        self._ready_signal.set()

    def _bind(
        self,
        dispatcher: "BotDispatcher",
        responder: "BotResponder",
        event_builder: Type["BotEventBuilder"],
        bot_bus: "BotHookBus",
        logger: BaseLogger,
    ) -> None:
        self._event_builder = event_builder
        self._bot_bus = bot_bus
        self.logger = logger
        self._common_dispatcher = dispatcher
        self._resp_dispatcher = responder

    @abstractmethod
    async def _send(self, action: "BotAction") -> None:
        raise NotImplementedError


class Flagable:
    def __init__(self) -> None:
        self._flags_store: Optional[dict[str, dict[str, Any]]] = None

    def mark(self, namespace: str, flag_name: str, val: Any = None) -> None:
        """为对象添加在指定命名空间下，名为 flag_name 的标记。 此后此对象会一直携带此标记，无法撤销。"""
        if self._flags_store is None:
            self._flags_store = {}
        if self._flags_store.get(namespace) is None:
            self._flags_store[namespace] = {}
        if flag_name in self._flags_store[namespace].keys():
            raise BotUtilsError(
                f"对象不可被重复标记。在命名空间 {namespace} 中名为 {flag_name} 的标记已存在"
            )
        self._flags_store[namespace][flag_name] = val

    def flag_check(self, namespace: str, flag_name: str, val: Any = None) -> bool:
        """检查此对象是否携带有指定的标记"""
        self._flags_store = self._flags_store
        if self._flags_store is None:
            return False
        if (flags := self._flags_store.get(namespace)) is None:
            return False
        if (flag := flags.get(flag_name, Void)) is Void:
            return False
        return flag is val if val is None else flag == val


class Cloneable:
    def copy(self):
        """返回一个本对象的一个深拷贝对象"""
        return deepcopy(self)


class BotEvent(ABC, Flagable):
    """事件基类

    .. admonition:: 提示
       :class: tip

       一般无需手动实例化该类，多数情况会直接使用本类对象，或将本类用作类型注解。
    """

    def __init__(self, rawEvent: dict) -> None:
        super().__init__()
        #: 从 onebot 实现项目获得的未格式化的事件原始值
        self.raw: dict = rawEvent
        self._args_map: Optional[dict[Any, dict[str, ParseArgs] | None]] = None

    def __format__(self, format_spec: str) -> str:
        match format_spec:
            case "hexid":
                return f"{id(self):#x}"
            case "raw":
                return self.raw.__str__()
            case _:
                raise BotValueError(f"未知的 event 格式标识符：{format_spec}")

    @property
    @abstractmethod
    def time(self) -> int:
        """事件发生的时间"""
        raise NotImplementedError

    @property
    @abstractmethod
    def type(self) -> Literal["message", "request", "notice", "meta", "response"]:
        """事件类型"""
        raise NotImplementedError

    def is_msg_event(self) -> bool:
        """判断是否是消息事件"""
        return self.type == "message"

    def is_req_event(self) -> bool:
        """判断是否是请求事件"""
        return self.type == "request"

    def is_notice_event(self) -> bool:
        """判断是否是通知事件"""
        return self.type == "notice"

    def is_meta_event(self) -> bool:
        """判断是否是元事件"""
        return self.type == "meta"

    def is_resp_event(self) -> bool:
        """判断是否是响应事件"""
        return self.type == "response"

    def _get_args(self, parser_id: Any) -> dict[str, ParseArgs] | VoidType | None:
        if self._args_map is None:
            return Void
        return self._args_map.get(parser_id, Void)

    def _store_args(
        self, parser_id: Any, args_dict: dict[str, ParseArgs] | None
    ) -> None:
        if self._args_map is None:
            self._args_map = {}
        self._args_map[parser_id] = args_dict


class ActionArgs(ABC):
    # 行为信息构造基类

    def __init__(self) -> None:
        super().__init__()
        self.type: str
        self.params: dict


class BotAction(Flagable, Cloneable):
    """行为类

    每个行为操作应该产生一个行为对象

    .. admonition:: 提示
       :class: tip

       一般无需手动实例化该类，多数情况会直接使用本类对象，或将本类用作类型注解。
    """

    def __init__(
        self,
        action_args: ActionArgs,
        resp_id: Optional[str] = None,
        triggerEvent: Optional[BotEvent] = None,
        ready: bool = False,
    ) -> None:
        super().__init__()
        self.resp_id = resp_id
        #: 行为类型。对应 onebot 标准中的 API 终结点名称
        self.type: str = action_args.type
        #: 行为参数。对应 onebot 标准中向 API 传送的数据的 params 字段
        self.params: dict = action_args.params
        #: 行为的触发事件，一般是行为生成时所在会话的事件
        self.trigger: Optional[BotEvent] = triggerEvent

        self._ready = ready

    def __format__(self, format_spec: str) -> str:
        match format_spec:
            case "hexid":
                return f"{id(self):#x}"
            case "raw":
                trigger = (
                    f"{self.trigger.__class__.__name__}[{self.trigger:hexid}]"
                    if self.trigger is not None
                    else None
                )
                return (
                    f"BotAction(type={self.type}, trigger={trigger}, "
                    f"resp_id={self.resp_id}, params={self.params})"
                )
            case _:
                raise BotValueError(f"未知的 action 格式标识符：{format_spec}")

    def extract(self) -> dict:
        # 从对象提取标准 cq action dict
        obj = {
            "action": self.type,
            "params": self.params,
        }
        if self.resp_id:
            obj["echo"] = self.resp_id
        return obj

    def flatten(self, indent: Optional[int] = None) -> str:
        # 将对象序列化为标准 cq action json 字符串，一般供连接器使用
        return json.dumps(self.extract(), ensure_ascii=False, indent=indent)

    def _fill_trigger(self, event: BotEvent) -> None:
        # 后期指定触发 event
        if self.trigger is None:
            self.trigger = event
            return
        raise BotRuntimeError("action 已记录触发它的 event，拒绝再次记录")


class SessionRule(ABC):
    """会话规则基类

    会话规则用于两事件是否在同一会话的判断。
    """

    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    def compare(self, e1: BotEvent, e2: BotEvent) -> bool:
        """判断两事件是否在同一会话中的判断方法

        任何会话规则应该实现此抽象方法。

        :param e1: 判断时的事件1
        :param e2: 判断时的事件2
        :return: 判断结果
        """
        raise NotImplementedError


class EventHandlerArgs:
    """事件方法（事件执行器）构造参数"""

    def __init__(
        self,
        executor: Callable[[], Coroutine[Any, Any, None]],
        type: Type["EventHandler"],
        params: list[Any],
    ) -> None:
        self.executor = executor
        if not is_retcoro(executor):
            raise BotValueError(
                f"事件处理函数 {executor.__qualname__} 必须为异步函数，或其他返回协程的可调用对象"
            )
        self.type = type
        self.params = params


class ShareObjArgs:
    """插件共享对象构造参数"""

    def __init__(
        self, namespace: str, id: str, reflector: Callable[[], Coroutine[Any, Any, Any]]
    ) -> None:
        if not is_retcoro(reflector):
            raise BotValueError(
                f"共享对象值获取函数 {reflector.__qualname__} 必须为异步函数，或其他返回协程的可调用对象"
            )
        self.reflector = reflector
        self.namespace = namespace
        self.id = id


class ShareObjCbArgs:
    """插件共享对象回调的构造参数"""

    def __init__(
        self, namespace: str, id: str, cb: Callable[..., Coroutine[Any, Any, Any]]
    ) -> None:
        if not is_retcoro(cb):
            raise BotValueError(
                f"共享对象的回调 {cb.__qualname__} 必须为异步函数，或其他返回协程的可调用对象"
            )
        self.namespace = namespace
        self.id = id
        self.cb = cb


class PluginSignalHandlerArgs:
    """插件信号方法构造参数"""

    def __init__(
        self, func: Callable[..., Coroutine[Any, Any, Any]], namespace: str, signal: str
    ) -> None:
        if not is_retcoro(func):
            raise BotValueError(
                f"插件信号处理函数 {func.__qualname__} 必须为异步函数，或其他返回协程的可调用对象"
            )
        self.func = func
        self.namespace = namespace
        self.signal = signal


class BotHookRunnerArgs:
    """钩子方法（生命周期回调）构造参数"""

    def __init__(
        self, func: Callable[..., Coroutine[Any, Any, None]], type: BotLife
    ) -> None:
        if not is_retcoro(func):
            raise BotValueError(
                f"bot 生命周期 hook {func.__qualname__} 必须为异步函数，或其他返回协程的可调用对象"
            )
        self.func = func
        self.type = type


class BotChecker(ABC, Cloneable):
    """检查器基类"""

    def __init__(
        self,
        ok_cb: Optional[Callable[[], Coroutine[Any, Any, None]]] = None,
        fail_cb: Optional[Callable[[], Coroutine[Any, Any, None]]] = None,
    ) -> None:
        """初始化一个检查器基类对象

        :param ok_cb: 检查通过的回调
        :param fail_cb: 检查不通过的回调
        """
        super().__init__()
        if ok_cb is not None and not is_retcoro(ok_cb):
            raise BotValueError(
                f"检查器成功回调 {ok_cb.__qualname__} 必须为异步函数，或其他返回协程的可调用对象"
            )
        if fail_cb is not None and not is_retcoro(fail_cb):
            raise BotValueError(
                f"检查器失败回调 {fail_cb.__qualname__} 必须为异步函数，或其他返回协程的可调用对象"
            )
        self.ok_cb = ok_cb
        self.fail_cb = fail_cb

    def __and__(self, other: "BotChecker") -> "WrappedChecker":
        if not isinstance(other, BotChecker):
            raise BotUtilsError(f"联合检查器定义时出现了非检查器对象，其值为：{other}")
        return WrappedChecker(LogicMode.AND, self, other)

    def __or__(self, other: "BotChecker") -> "WrappedChecker":
        if not isinstance(other, BotChecker):
            raise BotUtilsError(f"联合检查器定义时出现了非检查器对象，其值为：{other}")
        return WrappedChecker(LogicMode.OR, self, other)

    def __invert__(self) -> "WrappedChecker":
        return WrappedChecker(LogicMode.NOT, self)

    def __xor__(self, other: "BotChecker") -> "WrappedChecker":
        if not isinstance(other, BotChecker):
            raise BotUtilsError(f"联合检查器定义时出现了非检查器对象，其值为：{other}")
        return WrappedChecker(LogicMode.XOR, self, other)

    def _fill_ok_cb(self, ok_cb: Callable[[], Coroutine[Any, Any, None]]) -> None:
        """后期指定 ok_cb 回调"""
        if self.ok_cb is not None:
            raise BotUtilsError(f"ok_cb 回调已经被初始化，值为：{self.ok_cb}")
        self.ok_cb = ok_cb

    def _fill_fail_cb(self, fail_cb: Callable[[], Coroutine[Any, Any, None]]) -> None:
        """后期指定 fail_cb 回调"""
        if self.fail_cb is not None:
            raise BotUtilsError(f"fail_cb 回调已经被初始化，值为：{self.fail_cb}")
        self.fail_cb = fail_cb

    @abstractmethod
    async def check(self, event: BotEvent) -> bool:
        """检查器检查方法

        任何检查器应该实现此抽象方法。

        :param event: 给定的事件
        :return: 检查是否通过
        """
        raise NotImplementedError


class WrappedChecker(BotChecker):
    """合并检查器

    在两个 :class:`BotChecker` 对象间使用 | & ^ ~ 运算符即可返回合并检查器。

    .. admonition:: 提示
       :class: tip

       一般无需手动实例化该类，多数情况会直接使用本类对象，或将本类用作类型注解。
    """

    def __init__(
        self,
        mode: LogicMode,
        checker1: BotChecker,
        checker2: Optional[BotChecker] = None,
    ) -> None:
        """初始化一个合并检查器

        :param mode: 合并检查的逻辑模式
        :param checker1: 检查器1
        :param checker2: 检查器2
        """
        super().__init__()
        self.mode = mode
        self.c1, self.c2 = checker1, checker2

    def _fill_ok_cb(self, ok_cb: Callable[[], Coroutine[Any, Any, None]]) -> None:
        """后期指定 ok_cb 回调，注意此时是联合检查器， 因此将被自动应用到所包含的所有检查器"""
        super()._fill_ok_cb(ok_cb)
        self.c1._fill_ok_cb(ok_cb)
        self.c2._fill_ok_cb(ok_cb) if self.c2 else None

    def _fill_fail_cb(self, fail_cb: Callable[[], Coroutine[Any, Any, None]]) -> None:
        """后期指定 fail_cb 回调，注意此时是联合检查器， 因此将被自动应用到所包含的所有检查器"""
        super()._fill_fail_cb(fail_cb)
        self.c1._fill_fail_cb(fail_cb)
        self.c2._fill_fail_cb(fail_cb) if self.c2 else None

    async def check(self, event: BotEvent) -> bool:
        return LogicMode.calc(
            self.mode,
            await self.c1.check(event),
            (await self.c2.check(event)) if self.c2 is not None else None,
        )


class BotMatcher(ABC, Cloneable):
    """匹配器基类"""

    def __init__(self) -> None:
        """初始化一个检查器基类对象"""
        super().__init__()

    def __and__(self, other: "BotMatcher") -> "WrappedMatcher":
        if not isinstance(other, BotMatcher):
            raise BotUtilsError(f"联合匹配器定义时出现了非匹配器对象，其值为：{other}")
        return WrappedMatcher(LogicMode.AND, self, other)

    def __or__(self, other: "BotMatcher") -> "WrappedMatcher":
        if not isinstance(other, BotMatcher):
            raise BotUtilsError(f"联合匹配器定义时出现了非匹配器对象，其值为：{other}")
        return WrappedMatcher(LogicMode.OR, self, other)

    def __invert__(self) -> "WrappedMatcher":
        return WrappedMatcher(LogicMode.NOT, self)

    def __xor__(self, other: "BotMatcher") -> "WrappedMatcher":
        if not isinstance(other, BotMatcher):
            raise BotUtilsError(f"联合匹配器定义时出现了非匹配器对象，其值为：{other}")
        return WrappedMatcher(LogicMode.XOR, self, other)

    @abstractmethod
    def match(self, text: str) -> bool:
        """匹配器匹配方法

        任何匹配器应该实现此抽象方法。

        :param text: 消息事件的文本内容
        :return: 是否匹配
        """
        raise NotImplementedError


class WrappedMatcher(BotMatcher):
    """合并匹配器

    在两个 :class:`BotMatcher` 对象间使用 | & ^ ~ 运算符即可返回合并匹配器

    .. admonition:: 提示
       :class: tip

       一般无需手动实例化该类，多数情况会直接使用本类对象，或将本类用作类型注解。
    """

    def __init__(
        self,
        mode: LogicMode,
        matcher1: BotMatcher,
        matcher2: Optional[BotMatcher] = None,
    ) -> None:
        """初始化一个合并匹配器

        :param mode: 合并匹配的逻辑模式
        :param matcher1: 匹配器1
        :param matcher2: 匹配器2
        """
        super().__init__()
        self.mode = mode
        self.m1, self.m2 = matcher1, matcher2

    def match(self, text: str) -> bool:
        return LogicMode.calc(
            self.mode,
            self.m1.match(text),
            self.m2.match(text) if self.m2 is not None else None,
        )


class BotParser(ABC):
    """解析器基类

    解析器一般用作从消息文本中按规则批量提取参数
    """

    def __init__(self, id: Any) -> None:
        """初始化一个解析器

        :param id: 解析器解析规则的标识

           id 相同，意味着解析规则相同。一组解析规则相同的解析器，只有第一个会实际运行解析。其他后续解析器会复用这个解析结果。

           id 标识存在的意义是增强复用性。如果你不需要复用，id 值给定不同的随机值即可。
        """
        super().__init__()
        #: 解析器的 id，即解析规则的表示
        self.id: Any = id
        #: 是否需要格式化（此属性可在继承后进行修改，若为否，则不再进行格式化）
        self.need_format: bool = False

    @abstractmethod
    def parse(self, text: str) -> Optional[dict[str, ParseArgs]]:
        """解析方法

        任何解析器应该实现此抽象方法

        :param text: 消息文本内容
        :return: 解析结果

           因为一次解析可能会得到多组结果，结果为字典或空值。字典键为一组解析结果的识别标志（例如命令解析中的命令名），值为解析参数对象（象征一组解析结果）
        """
        raise NotImplementedError

    @abstractmethod
    def test(
        self, args_dict: Optional[dict[str, ParseArgs]]
    ) -> tuple[bool, Optional[str], Optional[ParseArgs]]:
        """解析测试方法

        任何解析器应该实现此抽象方法

        :param args_dict: 之前的解析结果
        :return: 返回元组：(判断是否解析成功, 可为空的一组解析结果的识别标志, 可为空的解析参数对象)
        """
        raise NotImplementedError

    @abstractmethod
    async def format(self, group_id: str, args: ParseArgs) -> bool:
        """格式化方法

        任何解析器应该实现此抽象方法

        格式化是否进行，会受解析器 `need_fotmat` 参数和 :meth:`~.BotParser.test` 的影响。
        如果你确定你的解析器子类 100% 不需要格式化，那么继承这个方法标记为 pass 即可。

        格式化过程最后只返回是否格式化成功，格式化的结果将会被直接保存，并可通过 :func:`.msg_args` 获得。

        :param group_id: 一组解析结果的识别标志
        :param args: 解析参数对象
        :return: 格式化是否成功
        """
        raise NotImplementedError



import io
import logging
import logging.config
import logging.handlers
import os
import sys
import types
from logging import CRITICAL

import colorlog
import rich.console
import rich.pretty
from better_exceptions import ExceptionFormatter

from ..base.abc import BaseLogger
from ..base.exceptions import BotValueError
from ..base.typing import Any, Callable, Literal, Optional


class NullLogger(BaseLogger):
    """获得一个空日志器，支持所有日志操作， 但是丢弃所有日志"""

    def __init__(self, name: str) -> None:
        super().__init__(name, CRITICAL)
        self.addHandler(logging.NullHandler())


class BotLogger(BaseLogger):
    """melobot 内置日志器类"""

    LOGGERS: dict[str, "BotLogger"] = {}

    LOG_COLORS = {
        "DEBUG": "cyan,bold",
        "INFO": "bold",
        "WARNING": "yellow,bold",
        "ERROR": "red,bold",
        "CRITIAL": "red,bold,bg_white",
    }

    SECOND_LOG_COLORS = {
        "message": {
            "DEBUG": "cyan",
            "WARNING": "yellow",
            "ERROR": "red",
            "CRITIAL": "red,bg_white",
        }
    }

    @staticmethod
    def _console_fmt(name: str, no_tag: bool = False) -> logging.Formatter:
        fmt_arr = [
            "%(green)s%(asctime)s.%(msecs)03d%(reset)s",
            "%(log_color)s%(levelname)-8s%(reset)s",
            "%(blue)s%(module)-10s%(reset)s : %(green)s%(lineno)-4d%(reset)s -> %(blue)s%(funcName)-12s%(reset)s",
            "%(message_log_color)s%(message)s%(reset)s",
        ]
        if not no_tag:
            fmt_arr.insert(0, f"%(purple)s{name}%(reset)s")
        fmt_s = f" {' │ '.join(fmt_arr)}"
        fmt = colorlog.ColoredFormatter(
            fmt_s,
            datefmt="%Y-%m-%d %H:%M:%S",
            log_colors=BotLogger.LOG_COLORS,
            secondary_log_colors=BotLogger.SECOND_LOG_COLORS,
            reset=True,
        )
        fmt.default_msec_format = "%s.%03d"
        return fmt

    @staticmethod
    def _console_handler(fmt: logging.Formatter) -> logging.Handler:
        handler = logging.StreamHandler()
        handler.setFormatter(fmt)
        return handler

    @staticmethod
    def _file_fmt(name: str, no_tag: bool = False) -> logging.Formatter:
        fmt_arr = [
            "%(asctime)s.%(msecs)03d",
            "%(levelname)-8s",
            "%(module)-10s : %(lineno)-4d -> %(funcName)-12s",
            "%(message)s",
        ]
        if not no_tag:
            fmt_arr.insert(0, name)
        fmt_s = " │ ".join(fmt_arr)
        fmt = logging.Formatter(
            fmt=fmt_s,
            datefmt="%Y-%m-%d %H:%M:%S",
        )
        fmt.default_msec_format = "%s.%03d"
        return fmt

    @staticmethod
    def _file_handler(
        fmt: logging.Formatter, log_dir: str, name: str
    ) -> logging.Handler:
        if not os.path.exists(log_dir):
            os.mkdir(log_dir)
        handler = logging.handlers.RotatingFileHandler(
            filename=os.path.join(log_dir, f"{name}.log"),
            maxBytes=1024 * 1024,
            backupCount=10,
            encoding="UTF-8",
        )
        handler.setFormatter(fmt)
        return handler

    def __init__(
        self,
        name: str,
        level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] = "INFO",
        to_console: bool = True,
        to_dir: Optional[str] = None,
        no_tag: bool = False,
    ) -> None:
        """初始化一个日志器实例

        :param name: 日志器的名称（唯一）
        :param level: 日志等级
        :param to_console: 是否输出到 console
        :param to_dir: 保存日志文件的目录，为空则不保存文件
        :param no_tag: 记录日志时是否不标识日志器名称
        """

        if name in BotLogger.LOGGERS.keys():
            raise BotValueError(f"名为 {name} 的日志器已存在，请修改 name")
        super().__init__(name, BotLogger.LEVEL_MAP[level])
        BotLogger.LOGGERS[name] = self
        self._con_handler: Optional[logging.Handler] = None
        self._handler_arr: list[logging.Handler] = []
        self._no_tag = no_tag

        if to_console:
            self._add_console_handler()
        if to_dir is not None:
            self._add_file_handler(to_dir)

    def _add_console_handler(self) -> None:
        if self._con_handler is None:
            self._con_handler = self._console_handler(
                self._console_fmt(self.name, self._no_tag)
            )
            self.addHandler(self._con_handler)
            self._handler_arr.append(self._con_handler)

    def _add_file_handler(self, log_dir: str) -> None:
        handler = self._file_handler(
            self._file_fmt(self.name, self._no_tag), log_dir, self.name
        )
        self.addHandler(handler)
        self._handler_arr.append(handler)

    def setLevel(self, level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]) -> None:  # type: ignore
        """设置日志等级

        日志等级自动应用于包含的所有 handler

        :param level: 日志等级字面量
        """
        super().setLevel(level)
        for handler in self._handler_arr:
            handler.setLevel(level)
        self.__LOG_LEVEL_FLAG__ = BotLogger.LEVEL_MAP[level]


_EXC_FORMATTER = ExceptionFormatter(colored=False)


def get_exc_stack(e: Exception) -> str:
    """返回生成更好的异常字符串"""
    return "".join(
        _EXC_FORMATTER.format_exception(e.__class__, e, sys.exc_info()[2])
    ).strip()


_CONSOLE_IO = io.StringIO()
_CONSOLE = rich.console.Console(file=_CONSOLE_IO)


def get_rich_str(obj: object, max_string: Optional[int] = 1000) -> str:
    """返回使用 rich 格式化的 object"""
    _CONSOLE.print(
        rich.pretty.Pretty(
            obj,
            indent_guides=True,
            max_string=max_string,
            overflow="ignore",
        ),
        crop=False,
    )
    string = _CONSOLE_IO.getvalue().strip("\n")
    _CONSOLE_IO.seek(0)
    _CONSOLE_IO.truncate(0)
    return string


def log_exc(
    logger: BaseLogger,
    locals: dict[str, Any],
    e: Exception,
) -> None:
    logger.error(f"异常回溯栈：\n{get_exc_stack(e)}")
    logger.error(f"异常抛出点局部变量：\n{get_rich_str(locals)}")


def log_obj(log_method: Callable[..., None], obj: Any, prefix: str) -> None:
    obj_str = f"{prefix}：\n{get_rich_str(obj)}"
    log_method(obj_str)


def logger_patch(
    logger: Any,
    log_level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] = "INFO",
) -> None:
    """修复任何非 :class:`.BotLogger` 类型的日志器，使其可以被用于 bot 实例初始化

    .. admonition:: 注意
       :class: caution

       非 :class:`.BotLogger` 类型的日志器，不使用该方法打补丁，无法被用于 bot 实例。但如果不用于 bot 实例，可以不打补丁。

    如果日志器的日志等级是可变的，请在更新后再次使用该方法修补

    :param logger: 任意日志器对象（支持 debug, info, warning, error, critical 方法即可）
    :param log_level: 日志器此时可以输出的最小日志等级（"DEBUG" 一端为小，"CRITICAL" 一端为大）
    """
    setattr(logger, BaseLogger.LEVEL_FLAG_NAME, BaseLogger.LEVEL_MAP[log_level])
    setattr(
        logger,
        BaseLogger.LEVEL_CHECK_METH_NAME,
        types.MethodType(BaseLogger.check_level_flag, logger),
    )



import asyncio

from ..base.abc import BaseLogger, BotLife
from ..base.exceptions import BotValueError
from ..base.typing import Any, Callable, Coroutine
from ..utils.logger import log_exc


class HookRunner:
    """bot hook 运行器"""

    def __init__(
        self, type: BotLife, func: Callable[..., Coroutine[Any, Any, None]]
    ) -> None:
        self.cb: Callable[..., Coroutine[Any, Any, None]] = func
        self.type: BotLife = type


class BotHookBus:
    """bot hook 总线"""

    def __init__(self) -> None:
        self.store: dict[BotLife, list[HookRunner]] = {
            v: [] for k, v in BotLife.__members__.items()
        }
        self.logger: BaseLogger

    def _bind(self, logger: BaseLogger) -> None:
        self.logger = logger

    def register(
        self, hook_type: BotLife, hook_func: Callable[..., Coroutine[Any, Any, None]]
    ) -> None:
        if hook_type not in self.store.keys():
            raise BotValueError(
                f"尝试添加一个 bot 生命周期 hook 方法，但是其指定的类型 {hook_type} 不存在"
            )
        runner = HookRunner(hook_type, hook_func)
        self.store[hook_type].append(runner)

    async def _run_on_ctx(self, runner: HookRunner, *args: Any, **kwargs: Any) -> None:
        try:
            await runner.cb(*args, **kwargs)
        except Exception as e:
            func_name = runner.cb.__qualname__
            self.logger.error(f"bot 生命周期 hook 方法 {func_name} 发生异常")
            log_exc(self.logger, locals(), e)

    async def emit(
        self, hook_type: BotLife, *args: Any, wait: bool = False, **kwargs: Any
    ) -> None:
        if not wait:
            for runner in self.store[hook_type]:
                asyncio.create_task(self._run_on_ctx(runner, *args, **kwargs))
        else:
            tasks = []
            for runner in self.store[hook_type]:
                tasks.append(
                    asyncio.create_task(self._run_on_ctx(runner, *args, **kwargs))
                )
            if len(tasks):
                await asyncio.wait(tasks)



def func(f):
    # 我嵌套定义一个函数
    def func2():
        print("f 准备执行！")
        f()
        print("f 结束执行！")

    return func2


@func
def test():
    print("这是原有的函数逻辑")


# 不再需要这样写
# new_f = func(test)
# new_f()

test()



import sys

import better_exceptions

# 修复在 windows powershell 显示错误的 bug
better_exceptions.encoding.ENCODING = sys.stdout.encoding
better_exceptions.formatter.ENCODING = sys.stdout.encoding
better_exceptions.hook()


class BotException(Exception):
    """Bot 异常基类

    在使用 melobot 编写代码时，若需要抛出异常，可以有意抛出该类。
    表明这是你设计的异常情况，而不是完全预期之外的异常。
    """

    def __init__(self, msg: str):
        super().__init__(self, msg)
        self.err = msg

    def __str__(self):
        return self.err


class BotValueError(BotException):
    def __init__(self, msg: str):
        super().__init__(msg)


class BotRuntimeError(BotException):
    def __init__(self, msg: str):
        super().__init__(msg)


class BotPluginError(BotException):
    def __init__(self, msg: str):
        super().__init__(msg)


class BotUtilsError(BotException):
    def __init__(self, msg: str):
        super().__init__(msg)


class BotIpcError(BotException):
    def __init__(self, msg: str):
        super().__init__(msg)


class BotSessionError(BotException):
    def __init__(self, msg: str):
        super().__init__(msg)


class BotSessionTimeout(BotException):
    """会话暂停的超时异常"""

    def __init__(self, msg: str = ""):
        super().__init__(msg)


class BotToolsError(BotException):
    def __init__(self, msg: str):
        super().__init__(msg)


class FuncSafeExited(BotException):
    def __init__(self, msg: str = ""):
        super().__init__(msg)
